;;;;
;;;;
cpu 8086
bits 16
org 100h
;;;;
;;;;

;;;;;;;;;;;;;;;; INSTALLER
_main:
	mov es, [2Ch]
	mov ah, 49h
	int 21h ; Free the environment segment.

	cli
	cld
	mov [2Ch], ss
	mov ss, [.UninstalledSegment] ; zero

	mov bp, 0x08*4
	mov bx, IRQ0
;;CHECKVECTOR BEGIN
	; INPUT:
	;  ss:bp -- pointer to interrupt vector
	;  ds:bx -- pointer to new interrupt handler (in cs)
	; TEMPS:  
	;  es:di -- pointer to current interrupt handler
	;  dx:ax -- previous vector (assuming inputter was installed)
	;  si    -- copy of bx (used while string-searching)
	;  dx:ax -- old handler address, if installing
	; UNUSED:
	;  cx,sp
	les di, [bp] ; current handler
	;lea si, [bx]
	mov si, bx
	cmpsw
	jne .VecNotSame 
	cmpsw
	jne .VecNotSame
	cmpsw
	jne .VecNotSame
	cmpsw
	jne .VecNotSame
.VecIsSame:
	; Uninstall the current vector occupant
	mov ax, [es:di-4-(4*2)+0]   ; Load previous vector (4*2 for 4*cmpsw)
	mov dx, [es:di-4-(4*2)+2]
	mov [.UninstalledSegment], es
	;mov [ss:bx],eax  ; store previous handler
	;ret
	; Below: almost the same, the redundant store to [bx-4] does not
	; matter
	jmp short .VecIdenticalCode
.InstalledText:
	db 'BASIC Syntax highlighting TSR installed',13,10
	db 'Copyright (C) 2010-12-10 Joel Yliluoma, http://iki.fi/bisqwit/',13,10
	db '$'
.VecNotSame:   
	; Store new handler address
	mov ax, bx
	mov dx, cs
.VecIdenticalCode:
	xchg ax, [bp]  ; store new handler, get old handler
	xchg dx, [bp+2]
	mov si, bx
	mov [si-4], ax
	mov [si-2], dx ; store old handler
;;CHECKVECTOR END
	mov ss, [2Ch]
	sti
	mov ax, word 0
.UninstalledSegment equ $-2
	or ax,ax
	jz .Installed
	mov es, ax
	mov ah, 49h ; Free the code segment
	int 21h
	mov dx, .UninstalledText
	mov ah, 9
	int 21h
	mov ax, 4C00h ; Exit
	int 21h
.Installed:
	mov dx, .InstalledText ; Display text
	mov ah, 9
	int 21h
	mov ax, 3100h
	mov dx, (((End - _main) + 15) >> 4) + 10h
	int 21h
.UninstalledText:
	db 'SYNHILI unloaded.',13,10
	db '$'

StackEnd: ; The installer region is not needed during IRQ, so we can reuse it as the stack.
;;;;;;;;;;;;;;;;;;;;; IRQ



%include "synhili.asm"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Position:  dw 0,0xB800
I08bulk:
	mov [cs:SavedSP], sp
	mov [cs:SavedSS], ss
	 push cs
	 pop ss
	 mov sp,StackEnd

	 ;pusha
	 push ax
	 push cx
	 push dx
	 push bx
	 push bp
	 push si
	 push di
	 
	 push ds
	 push es
	  call Highlight
	 pop es
	 pop ds

	 ;popa
	 pop di
	 pop si
	 pop bp
	 pop bx
	 pop dx
	 pop cx
	 pop ax
	 
	mov sp, 0
    SavedSS equ $-2
	mov ss, sp
	mov sp, 0
    SavedSP equ $-2
GotoOldI08:
	db 0EAh ; jmp far
OldI08: dw 0,0
IRQ0:	jmp I08bulk
	;push ax
	; mov al, 20h
	; out 20h, al
	;pop ax
	;iret

Highlight:
	xor ax,ax
	mov ds,ax
	mov ax, [ds:0x44A]
	cmp ax, 80
	jne Not80ColumnMode
	lds si, [cs:Position]
	cmp [24*80*2], byte '1'
	jne Not80ColumnMode ; Do highlighting only when KEY line is visible.
	cld
	mov bx, word 0  ; mov bx, Threshold
    Threshold equ $-2
	mov di, test_string_uc
	mov dx, takefrombuffer_uc
	mov cx, binarysearch
	lea ax, [si+80*2*5]	; process about 5 rows at once
	cmp ax, 24*80*2-2
	jbe .htmp
	 mov ax, 24*80*2-2
.htmp:	 mov [cs:Threshold], ax
	or bx, bx
	je st_newline
	jmp st_idle_entry

ThresholdExceeded:
	jmp EndHighlight

EndHighlight:
	cmp si, 24*80*2
	jb NotResetPosition
	xor si,si
NotResetPosition:
	mov [cs:Position], si
	ret
Not80ColumnMode:
	mov [cs:Position], ds ; happens to be 0
	mov [cs:Threshold], ds
	ret
;;;;;;;;;;;;;;;;;;;;; UTILITY

;;;;
test_string:
	pop bp        ; at [bp], is labels address
	push bp       ; ss conveniently points to cs, so we use bp here.
test_string_loop:
	lodsw
	cmp al, [bp+2]       ; 2 = string starting offset
	jne test_string_fail
	inc bp
	cmp si, bx
	jb test_string_loop
test_string_match:
	pop bp
	mov bp, [bp]
	jmp word [bp+2]
test_string_fail:
	pop bp
	mov bp, [bp]
	jmp word [bp+0]

;;;;
test_string_uc:
	pop bp        ; at [bp], is labels address
	push bp       ; ss conveniently points to cs, so we use bp here.
test_string_loop_uc:
	call dx ;;takefrombuffer_uc
	cmp al, [bp+2]       ; 2 = string starting offset
	jne test_string_fail
	inc bp
	cmp si, bx
	jb test_string_loop_uc
	jmp test_string_match

;;;;
takefrombuffer_uc:
	lodsw
makeucase:
	cmp al, 'a'
	jb notlc
	cmp al, 'z'
	ja notlc
	sub al, 32
notlc:	ret

;;;;
binarysearch:
	; don't use: si = source pointer
	;            bx = end pointer
	;            cx = string length
	;            al = input
	; can use:   bp
	;            di -if push&pop
	;            dx -if push&pop
	;            ah
	pop bp ; ss conveniently points to cs, so we use bp here.
	push di
	push dx
	mov dl, byte[bp]  ; dl=upper limit (exclusive), dh=lower limit
	mov dh, 0         ; Note: yes, they are vice versa. This was intentional. Before we outlawed movzx.
	jmp short .loop_assume_nonempty
.larger:
	inc ah
	mov dh, ah
	;; jmp .loop ;Redundant jump
.loop:
	cmp dh, dl
	jae .fail
.loop_assume_nonempty:
	mov ah, dh
	add ah, dl
	shr ah, 1
	mov di, ax ; movzx di, ah
	push cx
	 mov cl, 8
	 shr di, cl
	pop cx     ; end movzx
	cmp al, [bp+di+3]
	jnbe .larger
.smaller:
	mov dl, ah
	jne .loop
	;; jmp .match ;Redundant jump
.match:
	mov dl, byte[bp]
	mov dh, 0
	add di, di ; byte-to-word size
	add bp, dx ; skip over the character options
	lea bp, [bp+di+2]  ; bp + ah*2 + dl + 3    (3 = 2+1, +1 is below)
.fail:	pop dx
	pop di
	jmp word [bp+1]


;;;;
paint_token:
	lea si,[bx+1]
	push cx
	 mov cx, es
.loop:	 sub si, 2
	 mov [si], ah
	 loop .loop
	pop cx
	ret
test_newline:
	mov ax, si
	;mov cl, 80*2
	;div cl
	;test ah, ah
	push cx
	push dx
	 mov cx, 80*2
	 xor dx,dx
	 div cx
	 test dx, dx
	pop dx
	pop cx
	ret

test_space:
	cmp al,32
	je end_test_space
	cmp al,255
	je end_test_space
	or al,al
end_test_space:
	ret

End:



